@@ -97,6 +97,7 @@ cwb_compress_rdx <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_
 #'   files of the indexed corpus.
 #' @param vrt_dir Directory with input corpus files (verticalised format / file
 #'   ending *.vrt).
+#' @param encoding The encoding of the files to be encoded, defaults to "utf8".
 #' @param s_attributes A `list` of named `character` vectors to declare
 #'   structural attributes that shall be encoded. The names of the list are the
 #'   XML elements present in the corpus. Character vectors making up the list
@@ -132,7 +133,7 @@ cwb_compress_rdx <- function(corpus, p_attribute, registry = Sys.getenv("CORPUS_
 #' unlink(data_dir)
 #' unlink(file.path(Sys.getenv("CORPUS_REGISTRY"), "btmin"))
 #' }
-cwb_encode <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY"), data_dir, vrt_dir, p_attributes = c("word", "pos", "lemma"), s_attributes){
+cwb_encode <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY"), data_dir, vrt_dir, encoding = "utf8", p_attributes = c("word", "pos", "lemma"), s_attributes){
   
   s_attributes_noanno <- unlist(lapply(
     names(s_attributes),
@@ -152,8 +153,19 @@ cwb_encode <- function(corpus, registry = Sys.getenv("CORPUS_REGISTRY"), data_di
     regfile = file.path(registry, tolower(corpus)),
     data_dir = data_dir,
     vrt_dir = vrt_dir,
+    encoding = encoding,
     p_attributes = p_attributes,
     s_attributes_anno = s_attributes_anno,
     s_attributes_noanno = s_attributes_noanno
   )
 }
+
+#' Get CWB version
+#' 
+#' Get the CWB version used and available when compiling the source code.
+#' 
+#' @export
+#' @return A `numeric_version` object.
+#' @examples
+#' cwb_version()
+cwb_version <- function() as.numeric_version(.cwb_version())
\ No newline at end of file
